<?php

namespace Drupal\css_js_agg_retention\Asset;

use Drupal\Core\Asset\AssetCollectionGroupOptimizerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Base class for selective deletion of asset aggregates.
 *
 * Decorates the lazy asset collection optimizers to prevent deletion
 * of recent CSS/JS aggregates during cache rebuilds.
 */
abstract class BaseOptimizerSelectiveDelete implements AssetCollectionGroupOptimizerInterface {

  protected const MAX_AGE = 45 * 86400;

  /**
   * The decorated asset collection optimizer.
   */
  protected AssetCollectionGroupOptimizerInterface $inner;

  /**
   * The file system service.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The logger factory.
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * The config factory.
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new BaseOptimizerSelectiveDelete.
   *
   * @param \Drupal\Core\Asset\AssetCollectionGroupOptimizerInterface $inner
   *   The decorated asset collection optimizer.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    AssetCollectionGroupOptimizerInterface $inner,
    FileSystemInterface $file_system,
    LoggerChannelFactoryInterface $logger_factory,
    ConfigFactoryInterface $config_factory,
  ) {
    $this->inner = $inner;
    $this->fileSystem = $file_system;
    $this->loggerFactory = $logger_factory;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function optimize(array $assets, array $libraries) {
    return $this->inner->optimize($assets, $libraries);
  }

  /**
   * {@inheritdoc}
   */
  public function optimizeGroup(array $group): string {
    return $this->inner->optimizeGroup($group);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAll(): void {
    $dir = $this->getAssetsPath();

    if (!$this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY)) {
      // Nothing to do.
      return;
    }

    $now = \Drupal::time()->getRequestTime();
    $files = $this->fileSystem->scanDirectory($dir, '/.*/', ['recurse' => TRUE]);
    $removed = 0;
    $scanned = count($files);

    // Get configured retention period, fallback to default.
    $config = $this->configFactory->get('css_js_agg_retention.settings');
    $retention_days = $config->get('retention_days') ?? (static::MAX_AGE / 86400);
    $max_age = $retention_days * 86400;

    foreach ($files as $file) {
      if ($now - filemtime($file->uri) > $max_age) {
        $this->fileSystem->delete($file->uri);
        $removed++;
      }
    }

    $this->loggerFactory->get('css_js_agg_retention')->notice(
      'Purged @count stale asset files from @dir (scanned @scanned total)',
      [
        '@count' => $removed,
        '@dir' => $dir,
        '@scanned' => $scanned,
      ]
    );
  }

  /**
   * Gets the assets directory path.
   *
   * @return string
   *   The assets directory path.
   */
  abstract protected function getAssetsPath(): string;

}
