<?php

namespace Drupal\css_js_agg_retention\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure CSS/JS aggregation retention settings.
 */
class AggRetentionSettingsForm extends ConfigFormBase {

  /**
   * Default retention period in days.
   */
  protected const DEFAULT_DAYS = 45;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'css_js_agg_retention_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['css_js_agg_retention.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('css_js_agg_retention.settings');

    $form['retention_days'] = [
      '#type' => 'number',
      '#title' => $this->t('Retention period (days)'),
      '#description' => $this->t('Aggregated CSS and JS files newer than this many days will be kept when caches are rebuilt. Files older than this threshold will be deleted to free up disk space.'),
      '#default_value' => $config->get('retention_days') ?? static::DEFAULT_DAYS,
      '#min' => 1,
      '#max' => 365,
      '#required' => TRUE,
    ];

    $form['help'] = [
      '#type' => 'details',
      '#title' => $this->t('About this setting'),
      '#collapsed' => TRUE,
    ];

    $form['help']['description'] = [
      '#markup' => $this->t('<p>By default, Drupal deletes <em>all</em> aggregated CSS and JS files when caches are cleared (e.g., during deployments). This can cause performance issues on the first page loads after cache clears, as Drupal needs to regenerate all the aggregated files.</p><p>This module changes that behavior to only delete files older than the retention period you specify above. This keeps recent aggregates available, improving performance after cache clears.</p><p><strong>Recommended:</strong> Set this to a value that balances disk space usage with performance. 30-90 days is typically appropriate for most sites.</p>'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('css_js_agg_retention.settings')
      ->set('retention_days', (int) $form_state->getValue('retention_days'))
      ->save();

    $this->messenger()->addStatus($this->t('CSS/JS aggregation retention settings have been updated.'));

    parent::submitForm($form, $form_state);
  }

}
