<?php

namespace Drupal\Tests\css_js_agg_retention\Unit\Asset;

use Drupal\css_js_agg_retention\Asset\BaseOptimizerSelectiveDelete;
use Drupal\Core\Asset\AssetCollectionGroupOptimizerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Tests\UnitTestCase;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * @coversDefaultClass \Drupal\css_js_agg_retention\Asset\BaseOptimizerSelectiveDelete
 * @group css_js_agg_retention
 */
class BaseOptimizerSelectiveDeleteTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * Tests that deleteAll() respects the retention threshold.
   */
  public function testDeleteAllRespectsRetentionThreshold() {
    // Create mocks.
    $inner = $this->prophesize(AssetCollectionGroupOptimizerInterface::class);
    $fileSystem = $this->prophesize(FileSystemInterface::class);
    $loggerFactory = $this->prophesize(LoggerChannelFactoryInterface::class);
    $logger = $this->prophesize(LoggerChannelInterface::class);
    $configFactory = $this->prophesize(ConfigFactoryInterface::class);
    $config = $this->prophesize(ImmutableConfig::class);

    // Create a concrete test class that can be controlled for testing.
    $testClass = new class($inner->reveal(), $fileSystem->reveal(), $loggerFactory->reveal(), $configFactory->reveal()) extends BaseOptimizerSelectiveDelete {

      /**
       * Test file timestamps for mocking filemtime behavior.
       *
       * @var array
       */
      private $testFiles = [];

      /**
       * Current time for testing.
       *
       * @var int
       */
      private $testTime;

      /**
       * {@inheritdoc}
       */
      protected function getAssetsPath(): string {
        return 'test://assets';
      }

      /**
       * Set test file timestamps for controlled testing.
       */
      public function setTestFiles(array $files): void {
        $this->testFiles = $files;
      }

      /**
       * Set test time for controlled testing.
       */
      public function setTestTime(int $time): void {
        $this->testTime = $time;
      }

      /**
       * Override deleteAll to use test data instead of real dependencies.
       */
      public function deleteAll(): void {
        $dir = $this->getAssetsPath();

        if (!$this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY)) {
          return;
        }

        $now = $this->testTime ?? time();
        $files = $this->fileSystem->scanDirectory($dir, '/.*/', ['recurse' => TRUE]);
        $removed = 0;
        $scanned = count($files);

        $config = $this->configFactory->get('css_js_agg_retention.settings');
        $retention_days = $config->get('retention_days') ?? (static::MAX_AGE / 86400);
        $max_age = $retention_days * 86400;

        foreach ($files as $file) {
          // Use test data instead of real filemtime().
          $file_time = $this->testFiles[$file->uri] ?? $now;
          if ($now - $file_time > $max_age) {
            $this->fileSystem->delete($file->uri);
            $removed++;
          }
        }

        $this->loggerFactory->get('css_js_agg_retention')->notice(
          'Purged @count stale asset files from @dir (scanned @scanned total)',
          [
            '@count' => $removed,
            '@dir' => $dir,
            '@scanned' => $scanned,
          ]
        );
      }

    };

    // Mock file system behavior.
    $fileSystem->prepareDirectory('test://assets', FileSystemInterface::CREATE_DIRECTORY)
      ->willReturn(TRUE);

    // Create test files with different timestamps.
    $now = time();
    $recent_file = (object) ['uri' => 'test://assets/recent.css'];
    $old_file = (object) ['uri' => 'test://assets/old.css'];

    $fileSystem->scanDirectory('test://assets', '/.*/', ['recurse' => TRUE])
      ->willReturn([$recent_file, $old_file]);

    // Set up test time and file timestamps.
    $testClass->setTestTime($now);
    $testClass->setTestFiles([
    // 1 day old.
      'test://assets/recent.css' => $now - 86400,
    // 50 days old.
      'test://assets/old.css' => $now - (50 * 86400),
    ]);

    // Mock configuration.
    $configFactory->get('css_js_agg_retention.settings')->willReturn($config->reveal());
    $config->get('retention_days')->willReturn(45);

    // Expect only the old file to be deleted.
    $fileSystem->delete($old_file->uri)->shouldBeCalledOnce();
    $fileSystem->delete($recent_file->uri)->shouldNotBeCalled();

    // Mock logger.
    $loggerFactory->get('css_js_agg_retention')->willReturn($logger->reveal());
    $logger->notice('Purged @count stale asset files from @dir (scanned @scanned total)', [
      '@count' => 1,
      '@dir' => 'test://assets',
      '@scanned' => 2,
    ])->shouldBeCalledOnce();

    // Execute the method.
    $testClass->deleteAll();
  }

}
